"""Workflow for validating an existing Edge port subscription."""

from typing import Any

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.utils.errors import ProcessFailureError
from orchestrator.workflow import StepList, begin, done, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products.product_types.edge_port import EdgePort
from gso.services.lso_client import LSOState, anonymous_lso_interaction
from gso.services.netbox_client import NetboxClient


@step("Prepare required keys in state")
def prepare_state(subscription_id: UUIDstr) -> State:
    """Add required keys to the state for the workflow to run successfully."""
    edge_port = EdgePort.from_subscription(subscription_id)

    return {"subscription": edge_port}


@step("Verify NetBox entries")
def verify_netbox_entries(subscription: EdgePort) -> None:
    """Validate required entries for an edge port in NetBox."""
    nbclient = NetboxClient()
    netbox_errors = []

    #  Raises en exception when not found.
    lag = nbclient.get_interface_by_name_and_device(
        subscription.edge_port.edge_port_name, subscription.edge_port.node.router_fqdn
    )
    if lag.description != str(subscription.subscription_id):
        netbox_errors.append(
            f"Incorrect description for '{lag}', expected "
            f"'{subscription.subscription_id}' but got '{lag.description}'"
        )
    if not lag.enabled:
        netbox_errors.append(f"NetBox interface '{lag}' is not enabled.")
    for member in subscription.edge_port.edge_port_ae_members:
        interface = nbclient.get_interface_by_name_and_device(
            member.interface_name, subscription.edge_port.node.router_fqdn
        )
        if interface.description != str(subscription.subscription_id):
            netbox_errors.append(
                f"Incorrect description for '{member.interface_name}', expected "
                f"'{subscription.subscription_id}' but got '{interface.description}'"
            )
        if not interface.enabled:
            netbox_errors.append(f"NetBox interface '{member.interface_name}' is not enabled.")

    if netbox_errors:
        raise ProcessFailureError(message="NetBox misconfiguration(s) found", details=str(netbox_errors))


@step("Check base config for drift")
def verify_base_config(subscription: dict[str, Any]) -> LSOState:
    """Workflow step for running a playbook that checks whether base config has drifted."""
    return {
        "playbook_name": "gap_ansible/playbooks/edge_port.yaml",
        "inventory": {"all": {"hosts": {subscription["edge_port"]["node"]["router_fqdn"]: None}}},
        "extra_vars": {
            "dry_run": True,
            "subscription": subscription,
            "verb": "create",
            "is_verification_workflow": "true",
        },
    }


@workflow(
    "Validate Edge Port Configuration", target=Target.SYSTEM, initial_input_form=wrap_modify_initial_input_form(None)
)
def validate_edge_port() -> StepList:
    """Validate an existing, active Edge port subscription.

    * Check correct configuration of interfaces in NetBox.
    * Verify create Edge port configuration.
    """
    return (
        begin
        >> store_process_subscription(Target.SYSTEM)
        >> prepare_state
        >> verify_netbox_entries
        >> anonymous_lso_interaction(verify_base_config)
        >> resync
        >> done
    )
